#include "main.h"
//==============================================================
extern const uint8_t ulp_main_bin_start[] asm("_binary_ulp_main_bin_start");
extern const uint8_t ulp_main_bin_end[]   asm("_binary_ulp_main_bin_end");
//==============================================================
static void init_ulp_program(void)
{
  esp_err_t err = ulp_load_binary(0, ulp_main_bin_start,
          (ulp_main_bin_end - ulp_main_bin_start) / sizeof(uint32_t));
  ESP_ERROR_CHECK(err);

  gpio_num_t gpio_one_wire_num = GPIO_NUM_4;
  int rtcio_one_wire_num = rtc_io_number_get(gpio_one_wire_num);
  assert(rtc_gpio_is_valid_gpio(gpio_one_wire_num) && "GPIO must be an RTC IO");

  rtc_gpio_init(gpio_one_wire_num);
  rtc_gpio_set_direction(gpio_one_wire_num, RTC_GPIO_MODE_INPUT_ONLY);

  uint32_t bit = rtcio_one_wire_num + 14;

  ulp_set_out_one_wire_pin&=0xf003ffff; // mask off bits 18-27
  ulp_set_out_one_wire_pin|=(bit<<18)|(bit<<23); // modify from and to bit to read
  ulp_set_in_one_wire_pin&=0xf003ffff; // mask off bits 18-27
  ulp_set_in_one_wire_pin|=(bit<<18)|(bit<<23); // modify from and to bit to read
  ulp_set_one_wire_pin&=0xf003ffff; // mask off bits 18-27
  ulp_set_one_wire_pin|=(bit<<18)|(bit<<23); // modify from and to bit to read
  ulp_clear_one_wire_pin&=0xf003ffff; // mask off bits 18-27
  ulp_clear_one_wire_pin|=(bit<<18)|(bit<<23); // modify from and to bit to read
  ulp_get_pin0&=0xf003ffff;          // mask off bits 18-27
  ulp_get_pin0|=(bit<<18)|(bit<<23); // modify from and to bit to read

#if CONFIG_IDF_TARGET_ESP32
  rtc_gpio_isolate(GPIO_NUM_12);
  rtc_gpio_isolate(GPIO_NUM_15);
#endif // CONFIG_IDF_TARGET_ESP32
  esp_deep_sleep_disable_rom_logging(); // suppress boot messages
  ulp_set_wakeup_period(0, 20000);
  err = ulp_run((&ulp_entry - RTC_SLOW_MEM) / sizeof(uint32_t)); //Start the program
  ESP_ERROR_CHECK(err);
}
//==============================================================
void app_main(void)
{
  esp_sleep_wakeup_cause_t cause = esp_sleep_get_wakeup_cause();
  printf("cause %d\n", cause);
  if (cause != ESP_SLEEP_WAKEUP_ULP)
  {
    printf("Not ULP wakeup, initializing ULP\n");
    init_ulp_program();
  }
  else
  {
    printf("ULP wakeup\n");
    printf("R0: %5d\n", ulp_dataR0 & UINT16_MAX);
  }
  printf("Entering deep sleep\n\n");
  ESP_ERROR_CHECK(esp_sleep_enable_ulp_wakeup());
  esp_deep_sleep_start();
}
//==============================================================
